@@ -48,10 +48,7 @@ supported_dplyr_methods <- list(
   group_vars = NULL,
   group_by_drop_default = NULL,
   ungroup = NULL,
-  mutate = c(
-    "window functions (e.g. things that require aggregation within groups)",
-    "not currently supported"
-  ),
+  mutate = NULL,
   transmute = NULL,
   arrange = NULL,
   rename = NULL,
@@ -66,12 +63,12 @@ supported_dplyr_methods <- list(
   compute = NULL,
   collapse = NULL,
   distinct = "`.keep_all = TRUE` not supported",
-  left_join = "the `copy` and `na_matches` arguments are ignored",
-  right_join = "the `copy` and `na_matches` arguments are ignored",
-  inner_join = "the `copy` and `na_matches` arguments are ignored",
-  full_join = "the `copy` and `na_matches` arguments are ignored",
-  semi_join = "the `copy` and `na_matches` arguments are ignored",
-  anti_join = "the `copy` and `na_matches` arguments are ignored",
+  left_join = "the `copy` argument is ignored",
+  right_join = "the `copy` argument is ignored",
+  inner_join = "the `copy` argument is ignored",
+  full_join = "the `copy` argument is ignored",
+  semi_join = "the `copy` argument is ignored",
+  anti_join = "the `copy` argument is ignored",
   count = NULL,
   tally = NULL,
   rename_with = NULL,
