@@ -334,6 +334,12 @@ test_that("dim method returns the correct number of rows and columns", {
   expect_identical(dim(ds), c(20L, 7L))
 })
 
+test_that("dimnames, colnames on Dataset objects", {
+  ds <- open_dataset(dataset_dir, partitioning = schema(part = uint8()))
+  col_names <- c("int", "dbl", "lgl", "chr", "fct", "ts", "part")
+  expect_identical(dimnames(ds), list(NULL, col_names))
+  expect_identical(colnames(ds), col_names)
+})
 
 test_that("dim() correctly determine numbers of rows and columns on arrow_dplyr_query object", {
   ds <- open_dataset(dataset_dir, partitioning = schema(part = uint8()))
@@ -904,6 +910,7 @@ test_that("Dataset and query print methods", {
     print(ds),
     paste(
       "FileSystemDataset with 2 Parquet files",
+      "8 columns",
       "int: int32",
       "dbl: double",
       "lgl: bool",
