@@ -17,6 +17,8 @@
 
 #pragma once
 
+#include <google/cloud/credentials.h>
+#include <google/cloud/options.h>
 #include <google/cloud/status.h>
 #include <google/cloud/storage/object_metadata.h>
 #include <google/cloud/storage/well_known_headers.h>
@@ -30,8 +32,17 @@
 
 namespace arrow {
 namespace fs {
+struct GcsOptions;
+
 namespace internal {
 
+struct GcsCredentialsHolder {
+  // Constructor needed for make_shared
+  explicit GcsCredentialsHolder(std::shared_ptr<google::cloud::Credentials> credentials)
+      : credentials(std::move(credentials)) {}
+  std::shared_ptr<google::cloud::Credentials> credentials;
+};
+
 ARROW_EXPORT Status ToArrowStatus(const google::cloud::Status& s);
 
 ARROW_EXPORT int ErrnoFromStatus(const google::cloud::Status& s);
@@ -53,6 +64,8 @@ ARROW_EXPORT Result<std::shared_ptr<const KeyValueMetadata>> FromObjectMetadata(
 
 ARROW_EXPORT std::int64_t Depth(std::string_view path);
 
+ARROW_EXPORT google::cloud::Options AsGoogleCloudOptions(const GcsOptions& options);
+
 }  // namespace internal
 }  // namespace fs
 }  // namespace arrow
