@@ -15,8 +15,7 @@
 // specific language governing permissions and limitations
 // under the License.
 
-// Interfaces for defining middleware for Flight clients. Currently
-// experimental.
+// Interfaces for defining middleware for Flight clients.
 
 #include "arrow/flight/cookie_internal.h"
 #include "arrow/flight/client.h"
@@ -65,7 +64,7 @@ size_t CaseInsensitiveHash::operator()(const std::string& key) const {
   return std::hash<std::string>{}(upper_string);
 }
 
-Cookie Cookie::Parse(const std::string_view& cookie_header_value) {
+Cookie Cookie::Parse(std::string_view cookie_header_value) {
   // Parse the cookie string. If the cookie has an expiration, record it.
   // If the cookie has a max-age, calculate the current time + max_age and set that as
   // the expiration.
@@ -159,8 +158,8 @@ CookiePair Cookie::ParseCookieAttribute(const std::string& cookie_header_value,
   }
 
   // Key/Value may be URI-encoded.
-  out_key = arrow::internal::UriUnescape(out_key);
-  out_value = arrow::internal::UriUnescape(out_value);
+  out_key = arrow::util::UriUnescape(out_key);
+  out_value = arrow::util::UriUnescape(out_value);
 
   // Strip outer quotes on the value.
   if (out_value.size() >= 2 && out_value[0] == '"' &&
