@@ -58,9 +58,10 @@ AesEncryptor::AesEncryptor(ParquetCipher::type alg_id, int key_len, bool metadat
 
 class AesDecryptor::AesDecryptorImpl {};
 
-int AesDecryptor::Decrypt(const uint8_t* plaintext, int plaintext_len, const uint8_t* key,
-                          int key_len, const uint8_t* aad, int aad_len,
-                          uint8_t* ciphertext) {
+int AesDecryptor::Decrypt(::arrow::util::span<const uint8_t> ciphertext,
+                          ::arrow::util::span<const uint8_t> key,
+                          ::arrow::util::span<const uint8_t> aad,
+                          ::arrow::util::span<uint8_t> plaintext) {
   ThrowOpenSSLRequiredException();
   return -1;
 }
@@ -91,7 +92,12 @@ std::shared_ptr<AesDecryptor> AesDecryptor::Make(
   return NULLPTR;
 }
 
-int AesDecryptor::CiphertextSizeDelta() {
+int AesDecryptor::PlaintextLength(int ciphertext_len) const {
+  ThrowOpenSSLRequiredException();
+  return -1;
+}
+
+int AesDecryptor::CiphertextLength(int plaintext_len) const {
   ThrowOpenSSLRequiredException();
   return -1;
 }
@@ -114,4 +120,6 @@ void QuickUpdatePageAad(int32_t new_page_ordinal, std::string* AAD) {
 
 void RandBytes(unsigned char* buf, int num) { ThrowOpenSSLRequiredException(); }
 
+void EnsureBackendInitialized() {}
+
 }  // namespace parquet::encryption
